/*

  disasmdump
  copyright (C) bytepointer.com
  disassembler routines copyright (c) 2007-2013 Oleh Yuschuk
  
  View changelist history is globals.h

  LICENSE NOTE: Excluding the source files (c) Oleh Yuschuk (part of the disassembler
  engine), I wrote the routines for these source files either specifically for this
  edition of the project or pulled them from one of my personal code libraries
   whose source code is not currently public. All routines present however form an Open-
  Source project in their entirety.  Disasmdump is free software: you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  version 3 as published by the Free Software Foundation. This program is distributed
  as is, in the hope that it will be useful, but WITHOUT ANY WARRANTY; Glitch or no
  glitch, there is not even an implied warranty of MERCHANTABILITY or FITNESS for any
  endeavor or particular purpose.

  See the GNU General Public License for more details.
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  
  ---
  
  If you have a question, feedback or would like to submit a bug, please visit
  bytepointer.com, or send a direct e-mail to admin@.
  
  Thank you for your interest!

*/
#include "cmdline.h"
#include <io.h>         //_get_osfhandle()


//
// Options::initColorScheme()
//
void Options::initColorScheme(void)
{
    //init color scheme
    ZEROMEM(m_cs);

    //configure the color scheme's foreground attributes
    m_cs.wDefault       = FG_WHITE;         //line default
    m_cs.wAddrCol       = FG_GRAY;          //column: addresses
    m_cs.wOpCodeCol     = FG_WHITE;         //column: opcodes
    m_cs.wOpCodePartial = FG_GRAY;          //column: opcodes - opcodes used to complete partial instruction, but not part of disassembly size (-s)
    m_cs.wDisDefault    = FG_LT_GREEN;      //disasm: default color

    m_cs.wDisPlain      = m_cs.wDisDefault; //disasm: plain commands
    
    m_cs.wDisOpImm      = FG_LT_RED;        //disasm:opcode: immediate/constant
    m_cs.wDisOpMemStack = FG_BROWN;         //disasm:opcode: memory Memory accessed over ESP or EBP
    m_cs.wDisOpMem      = FG_BROWN;         //disasm:opcode: memory other
    m_cs.wDisOpReg      = FG_YELLOW;        //disasm:opcode: register
    m_cs.wDisOpRegFpu   = FG_LT_PURPLE;     //disasm:opcode: FPU, MMX and SSE registers
    m_cs.wDisOpRegSys   = FG_GRAY;          //disasm:opcode: Segment and system registers
    
    m_cs.wDisInsCall    = FG_LT_CYAN;       //disasm:instruction:branch: call, int
    m_cs.wDisInsRet     = FG_LT_CYAN;       //disasm:instruction:branch: ret
    m_cs.wDisInsBranchC = FG_LT_CYAN;       //disasm:instruction:branch: conditional branch (Jc, ...)
    m_cs.wDisInsBranchU = FG_LT_CYAN;       //disasm:instruction:branch: unconditional branch (jmp)
    
    m_cs.wDisInsStack   = m_cs.wDisDefault; //disasm:instruction: stack (i.e. push/pop)
    m_cs.wDisInsFpu     = FG_PURPLE;        //disasm:instruction: FPU, MMX, 3DNow! and SSE commands
    m_cs.wDisInsSys     = FG_GRAY;          //disasm:instruction: Bad, system and privileged commands
    
    m_cs.wDisBrackets   = FG_WHITE;         //disasm: brackets
    m_cs.wDisOp         = FG_GRAY;          //disasm: operators and symbols: :,+*

} //Options::initColorScheme()

//
// cmdlineShowUsage()
//
static int cmdlineShowUsage(const Options& options, int argc, const TCHAR* const* argv)
{
    return
    (
        _tprintf
        (
            APP_GREETING_LINE SZLF
            APP_COPYRIGHT_LINE SZLF
            VER_COMMENTS SZLF
            SZLF
            GPL_DISCLAIMER_LINES
            SZLF
            TEXT("syntax:") SZLF
            TEXT("  %s [options] [data_bytes]") SZLF
            SZLF
            TEXT("[data_bytes]") SZLF
            TEXT("  data may be specified directly as zero-padded hex digits, where two hex") SZLF
            TEXT("  digits delineate each byte - spaces optional; e.g.: 90 33 C0 83 C4 04") SZLF
            SZLF
            //    XXXXXXXXXXXXXXXXXXXXXXXXX 80 CHARACTER COLUMN XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
            TEXT("[options]") SZLF
            SZLF
            TEXT("  **VISUAL**") SZLF
            TEXT("  -v        - enable verbose output (PE info and ending totals summary)") SZLF
            TEXT("  -b        - disassembler: show numeric constants as unprefixed base 10 values") SZLF
            TEXT("  -u        - disassembler: uppercase output") SZLF
            TEXT("  -g        - disassembler: show default segments") SZLF
            TEXT("  -1        - disable address column") SZLF
            TEXT("  -2        - disable opcode column") SZLF
            TEXT("  -3        - display character column (default=off)") SZLF
            TEXT("  -r        - disable colors") SZLF
            TEXT("  -n <uint> - indent each disassembly line by this amount") SZLF
            TEXT("  -c <uint> - specify opcode column size (default=%u)") SZLF
            TEXT("  -d <uint> - specify disassembly column size (default=%u); affects position of") SZLF
            TEXT("              char column when specified") SZLF
            SZLF
            TEXT("  **INPUT SELECTION**") SZLF
            TEXT("  -f <file> - disassemble contents of specified file instead of on cmdline") SZLF
            TEXT("  -p        - PE pointer mode; this mode requires an input file and alters the") SZLF
            TEXT("              the -a option behavior to take a pointer address instead of a") SZLF
            TEXT("              file offset; when -a is unspecified (or zero), disassembly begins") SZLF
            TEXT("              at the address of the module's code entry point.  In this mode") SZLF
            TEXT("              the maximum amount of bytes that can be disassembled is to the") SZLF
            TEXT("              end of the PE section that contains the address.") SZLF
            TEXT("  -a <uint> - start disassembling at specified offset of data; when the -p option") SZLF
            TEXT("              is used, this option specifies a pointer address into PE image.") SZLF
            TEXT("  -i <uint> - initialize disassembly address (eip) numbering with specified") SZLF
            TEXT("              address for display purposes; when -p is present, the eip value is") SZLF
            TEXT("              automatically determined and this option is ignored.") SZLF
            TEXT("  -s <uint> - only disassemble complete instructions up to this size;") SZLF
            TEXT("              by default, the disassembler attempts to disassemble all data as") SZLF
            TEXT("              it is sequentially encountered, even if the") SZLF
            TEXT("              last instruction results in a partial instruction (disassembly") SZLF
            TEXT("              instruction results in \"???\"); by specifying a smaller size") SZLF
            TEXT("              with this option, the trailing data may be used by the") SZLF
            TEXT("              disassembler to complete the last instruction if it is a partial") SZLF
            TEXT("              one; in the case of a partial instruction, the data size") SZLF
            TEXT("              disassembled would be slightly more than amount specified here;") SZLF
            TEXT("              this value cannot exceed the size of the passed data.") SZLF
            TEXT("  -?        - this help") SZLF
            SZLF
            TEXT("NOTE: numbers given for the options above default to base-10; to specify hex,") SZLF
            TEXT("binary or octal values, please prefix with \"0x\", \"0b\" or \"0\" respectively.") SZLF
            SZLF
            TEXT("Special thanks to to Oleh Yuschuk for publicly releasing the source code") SZLF
            TEXT("to the OllyDbg Disassembling Engine v2.01 from which this tool is based.") SZLF
            ,
            getFilenamePtr(argv[0]),
            options.m_uOpCodeColSize,
            options.m_uDisasmColSize
        )
    );

} //cmdlineShowUsage()

//
// handleCommandLineData() - this function performs two similar tasks:
//
//      -pass #1: for each command line argument, validates and counts the number of hex digits needed, thus is used to calculate buffer size
//      -pass #2: for each command line argument, each hex digit pair is converted into byte representation and stored at current position in buffer
//
static int handleCommandLineData(const TCHAR* pszArg, dword& uDataSize, byte* pDataBuf, byte** ppCurData)
{
    int iRetCode = CODE_SUCCESS;
    if (!pszArg || !*pszArg)
    {
        _tprintf(TEXT("INTERNAL ERROR: empty data argument\n"));
        iRetCode = CODE_ERROR_INTERNAL;
    }
    else
    {
        dword uHexDigits = 0;
        const TCHAR* p = pszArg;
        byte* d = NULL;
        
        //if data buffer passed, default to start of buffer (or current position)
        if (pDataBuf)
        {
            if (*ppCurData)
            {
                d = *ppCurData;
            }
            else
            {
                d = pDataBuf;
            }
        }
        
        //loop through string (ignoring spaces) to validate hex digits
        while (*p)
        {
            if (SPACE != *p)
            {
                TCHAR ch = TO_UPPER(*p);
                bool bIsNumber = ch>=TEXT('0') && ch<=TEXT('9');
                bool bIsLetter = ch>=TEXT('A') && ch<=TEXT('F');
                if (bIsNumber || bIsLetter)
                {
                    ++uHexDigits;
                    
                    //do we have a destination buffer?
                    if (pDataBuf)
                    {
                        //convert digits into binary byte value in current buffer position
                        if ( (size_t)(d-pDataBuf) >= uDataSize )
                        {
                            _tprintf(TEXT("INTERNAL ERROR: allocated data buffer exhausted at idx=%u (size=%u)\n"),d-pDataBuf,uDataSize);
                            iRetCode = CODE_ERROR_INTERNAL;
                            break;
                        }
                        else
                        {
                            byte uDigitVal = (byte)(bIsNumber?ch - TEXT('0'):ch - TEXT('A')+10);
                            if (IS_ODD(uHexDigits))
                            {
                                //first hex byte digit value is 1 power larger than one to right (16 times)
                                //*d = uDigitVal*16;
                                *d = uDigitVal<<4;
                            }
                            else
                            {
                            
                            #ifdef _MSC_VER
                                #pragma warning (push)
                                #pragma warning (disable:4244) //C4244: '+=' : conversion from 'int' to 'byte', possible loss of data
                            #endif
                            
                                //second and last hex digit value is simply added to first
                                *d += uDigitVal;
                                
                            #ifdef _MSC_VER
                                #pragma warning (pop)
                            #endif

                                ++d;
                            }
                        }
                                                
                    } //destination buffer passed / calculate binary value
                }
                else
                {
                    _tprintf(TEXT("ERROR: \"%c\" is not a hex digit\n"),ch);
                    iRetCode = CODE_ERROR_RANGE;
                    break;
                }
                
            } //character not a space

            ++p;

        } //while each data character
        
        if (CODE_SUCCEEDED(iRetCode))
        {
            //enforce an even number of hex digits, disallowing partial byte values or assumptions of what they could mean!
            //  2 hex digits per byte
            if (IS_ODD(uHexDigits))
            {
                _tprintf(TEXT("ERROR: \"%s\" is an odd number of hex digits; please zero-pad as necessary\n"),pszArg);
                iRetCode = CODE_ERROR_ARGUMENTS;
            }
            else if (!pDataBuf)
            {
                //when destination buffer not passed, we are only calculating buffer size needed
                //  half the number of hex digits is the number of byes we have
                uDataSize += uHexDigits >> 1;
            }
            else
            {
                //keep track of running data position
                *ppCurData = d;
            }
        }
        
    } //arguments validated
        
    return(iRetCode);
    
} //handleCommandLineData()

//
// loadFileIntoBuffer()
//
int loadFileIntoBuffer(const TCHAR* pszFilename, byte** ppDataBuf, dword* puDataSize)
{
    int iRetCode = CODE_SUCCESS;
    
    //open file
    FILE* pFile = _tfopen(pszFilename,TEXT("rb"));
    if (!pFile)
    {
        _tprintf(TEXT("ERROR: unable to open file \"%s\"\n"),pszFilename);
        iRetCode = CODE_ERROR_FILE_OPEN;
    }
    else
    {
        //get file size
        dword dwSizeHi = 0;
        dword dwSizeLo = GetFileSize((HANDLE)_get_osfhandle(_fileno(pFile)),&dwSizeHi);
        if (INVALID_FILE_SIZE == dwSizeLo)
        {
            _tprintf(TEXT("ERROR: unable to get file size of file \"%s\"\n"),pszFilename);
            iRetCode = CODE_ERROR_FILE_QUERY;
        }
        else if (dwSizeHi)
        {
            _tprintf(TEXT("ERROR: file sizes in excess 32-bits unsupported size=0x%X%08X\n"),dwSizeHi,dwSizeLo);
            iRetCode = CODE_ERROR_ARGUMENTS;
        }
        else
        {
            //allocate a buffer of said file size
            assert(!*ppDataBuf);
            byte* pBuffer = new byte[dwSizeLo];
            if (!pBuffer)
            {
                _tprintf(TEXT("ERROR: unable to allocate %u bytes for file buffer\n"),dwSizeLo);
                iRetCode = CODE_ERROR_ARGUMENTS;
            }
            else
            {
                //read entire file into buffer at once
                size_t uBytesRead = fread(pBuffer,1,dwSizeLo,pFile);
                if (uBytesRead != dwSizeLo)
                {
                    _tprintf(TEXT("ERROR: error reading \"%s\"; tried to read %u bytes but instead read %u\n"),pszFilename,dwSizeLo,uBytesRead);
                    iRetCode = CODE_ERROR_ARGUMENTS;
                    
                    //cleanup buffer if we hit a read error
                    delete[] pBuffer;
                    pBuffer = NULL;
                }
                else
                {
                    //return buffer back to caller (it now becomes callers responsibility to clean up)
                    *ppDataBuf = pBuffer;
                    *puDataSize = dwSizeLo;
                    
                } //file read into buffer successfully
                
            } //buffer allocated successfully
            
        } //file size read successfully
    
        //cleanup file
        fclose(pFile);

    } //file opened successfully

    return(iRetCode);

} //loadFileIntoBuffer()

//
// Options::processCommandLine()
//
int Options::processCommandLine(int argc, const TCHAR* const* argv)
{
    int iRetCode = CODE_SUCCESS;
    bool bShowHelp = false;
    assert(NULL == m_pDataBuf);
    
    if (1 >= argc)
    {
        //no arguments, show command line help
        bShowHelp = true;
        iRetCode = CODE_ERROR_ARGUMENTS;
    }
    else
    {
        //go through each command line argument
        for (int iArgIdx = 1; iArgIdx<argc && CODE_SUCCEEDED(iRetCode); ++iArgIdx)
        {
            //do we have a non-empty argument?
            const TCHAR* pszCurArg = argv[iArgIdx];
            bool bAtLastArg = (iArgIdx+1)>=argc;
            if (*pszCurArg)
            {
                //not a command line option, assume hex data
                if (TEXT('-') != *pszCurArg)
                {
                    if (m_pszFilename)
                    {
                        _tprintf(TEXT("ERROR: cannot combine file data with command line data\n"));
                        iRetCode = CODE_ERROR_ARGUMENTS;
                    }
                    else
                    {
                        //assume this is one or more hex byte arguments (when not separated by spaces)
                        //  at this point (phase #1) we are only counting them so we can later allocate a buffer to fit them
                        iRetCode = handleCommandLineData(pszCurArg,m_uDataSize,m_pDataBuf,NULL);
                    }
                }
                else
                {
                    bool bOptionsDone = false;
                    int iStartArg = iArgIdx;
                    bool bDisplayedError = false;

                    //do we recognize this option char?
                    do
                    {
                        //advance to next cmdline option character and test it
                        ++pszCurArg;
                        switch (*pszCurArg)
                        {
                            case TEXT('?'):
                                bShowHelp = true;
                                iRetCode = CODE_ERROR_ARGUMENTS;
                                break;

                            case TEXT('v'):
                                m_bVerbose = true;
                                break;

                            //leave this in for backwards compatibility, although it is now the default
                            case TEXT('x'):
                                m_bShowHexValues = true;
                                break;

                            case TEXT('b'):
                                m_bShowHexValues = false;
                                break;

                            case TEXT('u'):
                                m_bUppercase = true;
                                break;

                            case TEXT('g'):
                                m_bDefaultSeg = true;
                                break;

                            case TEXT('1'):
                                m_bWantColAddr = false;
                                break;

                            case TEXT('2'):
                                m_bWantColOpCodes = false;
                                break;

                            case TEXT('3'):
                                m_bWantColChars = true;
                                break;

                            case TEXT('r'):
                                m_bColor = false;
                                break;

                            case TEXT('p'):
                                m_bPeAddr = true;
                                break;

                            case TEXT('s'):
                                if (bAtLastArg)
                                {
                                    _tprintf(TEXT("ERROR: please specify value for -s\n"));
                                    iRetCode = CODE_ERROR_ARGUMENTS;
                                    bDisplayedError = true;
                                }
                                else
                                {
                                    bOptionsDone = true;
                                    iRetCode = stringToDword(argv[++iArgIdx],&m_uDisasmToSize);
                                }
                                break;

                            case TEXT('a'):
                                if (bAtLastArg)
                                {
                                    _tprintf(TEXT("ERROR: please specify value for -a\n"));
                                    iRetCode = CODE_ERROR_ARGUMENTS;
                                    bDisplayedError = true;
                                }
                                else
                                {
                                    bOptionsDone = true;
                                    iRetCode = stringToDword(argv[++iArgIdx],(dword*)&m_uStartOffset);
                                }
                                break;

                            case TEXT('i'):
                                if (bAtLastArg)
                                {
                                    _tprintf(TEXT("ERROR: please specify value for -i\n"));
                                    iRetCode = CODE_ERROR_ARGUMENTS;
                                    bDisplayedError = true;
                                }
                                else
                                {
                                    bOptionsDone = true;
                                    iRetCode = stringToDword(argv[++iArgIdx],&m_uInitAddr);
                                    m_bInitAddrSpecified = true;
                                }
                                break;

                            case TEXT('c'):
                                if (bAtLastArg)
                                {
                                    _tprintf(TEXT("ERROR: please specify value for -c\n"));
                                    iRetCode = CODE_ERROR_ARGUMENTS;
                                    bDisplayedError = true;
                                }
                                else
                                {
                                    bOptionsDone = true;
                                    iRetCode = stringToDword(argv[++iArgIdx],(dword*)&m_uOpCodeColSize);
                                    
                                    //why allow an opcode column greater than this amount?
                                    if (m_uOpCodeColSize > 50)
                                    {
                                        _tprintf(TEXT("ERROR: opcode column size out of range\n"));
                                        iRetCode = CODE_ERROR_RANGE;
                                        bDisplayedError = true;
                                    }
                                }
                                break;

                            case TEXT('d'):
                                if (bAtLastArg)
                                {
                                    _tprintf(TEXT("ERROR: please specify value for -d\n"));
                                    iRetCode = CODE_ERROR_ARGUMENTS;
                                    bDisplayedError = true;
                                }
                                else
                                {
                                    bOptionsDone = true;
                                    iRetCode = stringToDword(argv[++iArgIdx],(dword*)&m_uDisasmColSize);
                                    
                                    //why allow an opcode column greater than this amount?
                                    if (m_uOpCodeColSize > 50)
                                    {
                                        _tprintf(TEXT("ERROR: disasm column size out of range\n"));
                                        iRetCode = CODE_ERROR_RANGE;
                                        bDisplayedError = true;
                                    }
                                }
                                break;

                            case TEXT('n'):
                                if (bAtLastArg)
                                {
                                    _tprintf(TEXT("ERROR: please specify value for -n\n"));
                                    iRetCode = CODE_ERROR_ARGUMENTS;
                                    bDisplayedError = true;
                                }
                                else
                                {
                                    bOptionsDone = true;
                                    iRetCode = stringToDword(argv[++iArgIdx],(dword*)&m_uIndentSize);
                                    if (m_uIndentSize > MAX_INDENT_LEN)
                                    {
                                        _tprintf(TEXT("ERROR: indent size out of range\n"));
                                        iRetCode = CODE_ERROR_RANGE;
                                        bDisplayedError = true;
                                    }
                                }
                                break;

                            case TEXT('f'):
                                if (bAtLastArg)
                                {
                                    _tprintf(TEXT("ERROR: please specify a filename for -f\n"));
                                    iRetCode = CODE_ERROR_ARGUMENTS;
                                    bDisplayedError = true;
                                }
                                else if (m_uDataSize)
                                {
                                    _tprintf(TEXT("ERROR: cannot combine command line data with file data\n"));
                                    iRetCode = CODE_ERROR_ARGUMENTS;
                                    bDisplayedError = true;
                                }
                                else
                                {
                                    bOptionsDone = true;
                                    m_pszFilename = argv[++iArgIdx];
                                    iRetCode = loadFileIntoBuffer(m_pszFilename,&m_pDataBuf,&m_uDataSize);
                                }
                                break;
                                
                            case 0:
                                //NULL terminator hit
                                bOptionsDone = true;
                                break;
                                
                            default:
                                _tprintf(TEXT("ERROR: unrecognized option \"%s\"\n"),pszCurArg);
                                iRetCode = CODE_ERROR_ARGUMENTS;
                                bDisplayedError = true;

                        } //switch

                    //keep looking for options in this arg until error, or null terminator hit 
                    } while (CODE_SUCCEEDED(iRetCode) && !bOptionsDone);
                    
                    if (CODE_FAILED(iRetCode))
                    {
                        if (!bDisplayedError)
                        {
                            _tprintf(TEXT("ERROR: while parsing argument \"%s\" or the one after (code=%d)\n"),argv[iStartArg],iRetCode);
                        }
                    }
                    //did we process an option that took a parameter (iArgIdx increased), but there were characters past it?
                    else if (iStartArg != iArgIdx && *(pszCurArg+1))
                    {
                        _tprintf(TEXT("ERROR: extra option chars encountered in argument \"%s\" at \"%s\"; please specify non-parameter options first\n"),argv[iStartArg],pszCurArg+1);
                        iRetCode = CODE_ERROR_ARGUMENTS;
                    }

                } //beginning of switch found

            } //have a non-empty argument
            
        } //for each cmdline arg

        if (CODE_SUCCEEDED(iRetCode))
        {
            //if starting address not explicitly specified, but we have a nonzero offset
            //  set starting address at that offset
            if (!m_bInitAddrSpecified && m_uStartOffset)
            {
                m_uInitAddr = m_uStartOffset;
            }
            
            //hex data must have been specified
            if (!m_uDataSize)
            {
                _fputts(TEXT("ERROR: please specify hex data\n"),stdout);
                iRetCode = CODE_ERROR_ARGUMENTS;
            }
            //validate date offset is less than data size (unless in PE mode we have a pointer address)
            else if (!m_bPeAddr && (m_uStartOffset+m_uDisasmToSize)>m_uDataSize)
            {
                _tprintf(TEXT("ERROR: disassembly size %u at offset %u cannot exceed %u (datasize=%u)\n"),m_uDisasmToSize,m_uStartOffset,m_uDataSize-m_uStartOffset,m_uDataSize);
                iRetCode = CODE_ERROR_ARGUMENTS;
            }
            else
            {
                //if m_pDataBuf is non-null, we have allocated buffer from file
                if (m_pDataBuf)
                {
                    if (m_bPeAddr && m_bInitAddrSpecified)
                    {
                        _tprintf(TEXT("WARNING: eip 0x%X ignored; no need to initialize address in PE pointer mode\n"),m_uInitAddr);
                    }
                }
                else if (m_bPeAddr) //m_pDataBuf == NULL
                {
                    _tprintf(TEXT("ERROR: you must specify an input file in PE mode\n"));
                    iRetCode = CODE_ERROR_ARGUMENTS;
                }
                else
                {
                    //SO FAR SO GOOD - allocate and build data buffer from command line data
                    assert(NULL == m_pszFilename);
                    assert(m_uDataSize);
                    m_pDataBuf = new byte[m_uDataSize];
                    if (!m_pDataBuf)
                    {
                        _tprintf(TEXT("ERROR: allocation of %u data bytes failed\n"),m_uDataSize);
                        iRetCode = CODE_ERROR_ALLOCATION;
                    }
                    else
                    {
                        //PASS #2 of command line arg loop - loop a second time, but only pulling out data
                        //  into newly allocated buffer

                        //go through each command line argument
                        byte* pCurDest = NULL;
                        for (int iArgIdx = 1; iArgIdx<argc && CODE_SUCCEEDED(iRetCode); ++iArgIdx)
                        {
                            //do we have a non-empty argument?
                            const TCHAR* pszCurArg = argv[iArgIdx];
                            if (*pszCurArg)
                            {
                                //not a switch, assume hex data
                                if (TEXT('-') != *pszCurArg)
                                {
                                    //read hex data directly
                                    iRetCode = handleCommandLineData(pszCurArg,m_uDataSize,m_pDataBuf,&pCurDest);
                                }
                                else
                                {
                                    //simulate previous option analyzer loop except we only need to know
                                    //  to advance the options that specified their own arguments so the
                                    //  the hex data reader gets the same data to read as it previously counted
                                    //
                                    //  this seems kind of ugly and hacky, but the payoff is that we don't have to make a separate
                                    //  copy of the data before we combine it all into one byte buffer
                                    bool bOptionsDone = false;
                                    do
                                    {
                                        ++pszCurArg;
                                        switch (*pszCurArg)
                                        {
                                            case TEXT('s'):
                                            case TEXT('a'):
                                            case TEXT('i'):
                                            case TEXT('c'):
                                            case TEXT('d'):
                                            case TEXT('n'):
                                            case TEXT('f'):
                                                ++iArgIdx; //advance past options that take their own args
                                                //drop through to exit inner loop early
                                            case 0:
                                                //NULL terminator hit
                                                bOptionsDone = true;
                                                break;
                                        }
                                    } while (!bOptionsDone);

                                } //beginning of switch found

                            } //have a non-empty argument
                            
                        } //for each cmdline arg
                    
                    } //allocation of data buffer succeeded

                } //build data buffer from command line data
                
            } //arguments validated successfully
            
        } //initial pass of arguments parsed without error

    } //we have arguments
        
    //show help?  ... but don't change return code
    if (bShowHelp)
    {
        if (CODE_FAILED(cmdlineShowUsage(*this,argc,argv))) assert(0);
    }
    
    return(iRetCode);

} //processCommandLine()

