/*

  disasmdump
  copyright (C) bytepointer.com
  disassembler routines copyright (c) 2007-2013 Oleh Yuschuk
  
  View changelist history is globals.h

  LICENSE NOTE: Excluding the source files (c) Oleh Yuschuk (part of the disassembler
  engine), I wrote the routines for these source files either specifically for this
  edition of the project or pulled them from one of my personal code libraries
   whose source code is not currently public. All routines present however form an Open-
  Source project in their entirety.  Disasmdump is free software: you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  version 3 as published by the Free Software Foundation. This program is distributed
  as is, in the hope that it will be useful, but WITHOUT ANY WARRANTY; Glitch or no
  glitch, there is not even an implied warranty of MERCHANTABILITY or FITNESS for any
  endeavor or particular purpose.

  See the GNU General Public License for more details.
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  
  ---
  
  If you have a question, feedback or would like to submit a bug, please visit
  bytepointer.com, or send a direct e-mail to admin@.
  
  Thank you for your interest!

*/
#ifndef _CMDLINE_H_
#define _CMDLINE_H_

#include "globals.h"


//color code combinations used by SetConsoleTextAttribute()
#define FG_BLACK        0
#define FG_WHITE        (FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_BLUE)
#define FG_LT_WHITE     (FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_BLUE|FOREGROUND_INTENSITY)
#define FG_GRAY         (FOREGROUND_INTENSITY)
#define FG_RED          FOREGROUND_RED
#define FG_LT_RED       (FOREGROUND_RED|FOREGROUND_INTENSITY)
#define FG_GREEN        FOREGROUND_GREEN
#define FG_LT_GREEN     (FOREGROUND_GREEN|FOREGROUND_INTENSITY)
#define FG_BLUE         FOREGROUND_BLUE
#define FG_LT_BLUE      (FOREGROUND_BLUE|FOREGROUND_INTENSITY)
#define FG_PURPLE       (FOREGROUND_RED|FOREGROUND_BLUE)
#define FG_LT_PURPLE    (FOREGROUND_RED|FOREGROUND_BLUE|FOREGROUND_INTENSITY)
#define FG_BROWN        (FOREGROUND_RED|FOREGROUND_GREEN)
#define FG_YELLOW       (FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_INTENSITY)
#define FG_CYAN         (FOREGROUND_BLUE|FOREGROUND_GREEN)
#define FG_LT_CYAN      (FOREGROUND_BLUE|FOREGROUND_GREEN|FOREGROUND_INTENSITY)


//
// Output colorscheme
//
struct ColorScheme
{
    word wDefault;
    word wAddrCol;
    word wOpCodeCol;
    word wOpCodePartial;
    word wDisDefault;
    word wDisPlain;
    word wDisOpImm;
    word wDisOpMemStack;
    word wDisOpMem;
    word wDisOpReg;
    word wDisOpRegFpu;
    word wDisOpRegSys;
    word wDisInsCall;
    word wDisInsRet;
    word wDisInsBranchC;
    word wDisInsBranchU;
    word wDisInsStack;
    word wDisInsFpu;
    word wDisInsSys;
    word wDisBrackets;
    word wDisOp;
};

//
// Program Options
//
#define MAX_INDENT_LEN 100
struct Options
{
    //data
    bool            m_bInitAddrSpecified;
    bool            m_bVerbose;
    bool            m_bShowHexValues;
    bool            m_bUppercase;
    bool            m_bDefaultSeg;
    bool            m_bWantColAddr;
    bool            m_bWantColOpCodes;
    bool            m_bWantColChars;
    bool            m_bColor;
    bool            m_bPeAddr;
    dword           m_uDataSize;
    dword           m_uDisasmToSize;
    dword           m_uStartOffset;
    dword           m_uInitAddr;
    uint            m_uOpCodeColSize;
    uint            m_uDisasmColSize;
    uint            m_uIndentSize;
    byte*           m_pDataBuf;    
    const TCHAR*    m_pszFilename;
    ColorScheme     m_cs;

    //ctors/dtor
    Options(void)
        :
        m_bInitAddrSpecified(false),
        m_bVerbose(false),
        m_bShowHexValues(true),
        m_bUppercase(false),
        m_bDefaultSeg(false),
        m_bWantColAddr(true),
        m_bWantColOpCodes(true),
        m_bWantColChars(false),
        m_bPeAddr(false),
        m_bColor(true),
        m_uDataSize(0),
        m_uDisasmToSize(0),
        m_uStartOffset(0),
        m_uInitAddr(0),
        m_uOpCodeColSize(24),
        m_uDisasmColSize(32),
        m_uIndentSize(0),
        m_pDataBuf(NULL),
        m_pszFilename(NULL)
    {
        initColorScheme();
    }
    
    ~Options(void)
    {
        if (m_pDataBuf) delete[] m_pDataBuf;
        m_pDataBuf = NULL;
    }

    //utility functions
    int     processCommandLine      (int argc, const TCHAR* const* argv);
    void    initColorScheme         (void);
    
    //disable copy ctor and assignment operators w/o body as per ISO standard
    private: Options(const Options& rhs); Options& operator= (const Options& rhs);

}; //struct Options

#endif //_CMDLINE_H_