/*

  disasmdump
  copyright (C) bytepointer.com
  disassembler routines copyright (c) 2007-2013 Oleh Yuschuk
  
  View changelist history is globals.h

  LICENSE NOTE: Excluding the source files (c) Oleh Yuschuk (part of the disassembler
  engine), I wrote the routines for these source files either specifically for this
  edition of the project or pulled them from one of my personal code libraries
   whose source code is not currently public. All routines present however form an Open-
  Source project in their entirety.  Disasmdump is free software: you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  version 3 as published by the Free Software Foundation. This program is distributed
  as is, in the hope that it will be useful, but WITHOUT ANY WARRANTY; Glitch or no
  glitch, there is not even an implied warranty of MERCHANTABILITY or FITNESS for any
  endeavor or particular purpose.

  See the GNU General Public License for more details.
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  
  ---
  
  stdinc.h
  
  This primary header has been stripped down and refactored from its original version.
  The LINUX definitions among other features that do not pertain to the current
  project have been removed for clarity. Although I made significant effort to render
  this code as clean as possible for educational and public consumption, remnants, such
  as comments referring to missing features directly or indirectly may still remain.
  Please visit bytepointer.com, or send a direct e-mail to admin@ to submit a bug, or
  other feedback including questions you might have. Thank you for your interest!

*/
#ifndef _STDINC_H_
#define _STDINC_H_

//STRNUM macro - creates a string literal of the value of the numeric macro rather than its name;
//  this trick is the only standard way to perform such a task with the C preprocessor, and allows
//  one to generate a string from the result of a macro evaluation (such as the value of _MSC_VER rather than the name _MSC_VER);
//  1st level evaluates macro into corresponding value while the 2nd level turns number into quoted string
#define _STRNUM(num)        #num
#define STRNUM(num)         _STRNUM(num)

//NUMPASTE macro - pastes two numbers side by side to form a new number, not a string
//  like the STRNUM macro above, it also uses two levels of indirection, the first being the evaluator (turns any other macros into a number)
//  and the second level pasting the numbers side by side; this macro allows the arguments to be other macros that evaluate to numbers
//  where the desired result is the evaluated numbers, not the names of the macros of those numbers; e.g.:
//
//  #define VER_MAJOR 1
//  #define VER_MINOR 5
//  NUMPASTE(VER_MAJOR,VER_MINOR)   //turns into "15"
//
#define _NUMPASTE(x,y)      x##y
#define NUMPASTE(x,y)       _NUMPASTE(x,y)

//if any of _DEBUG or DEBUG are defined, ensure both are defined
#if defined(_DEBUG) || defined(DEBUG)
    #ifndef _DEBUG
        #define _DEBUG
    #endif
    #ifndef DEBUG
        #define DEBUG
    #endif
#endif

//if any of _WIDE or WIDE are defined, ensure both are defined
#if defined(_WIDE) || defined(WIDE)
    #ifndef _WIDE
        #define _WIDE
    #endif
    #ifndef WIDE
        #define WIDE
    #endif
#endif

#if defined(WIN32) || defined(_WIN32) || defined(WIN64) || defined(_WIN64)
    //#define WINDOWS to simplify define logic just above
    #ifndef WINDOWS
        #define WINDOWS
    #endif

    //declare function attribute identifiers with GCC
    #define STDCALL __stdcall
    #define CDECL   __cdecl

    //if any of _WIN32 or WIN32 is defined, ensure both are defined
    #if defined(_WIN32) || defined(WIN32)
        #ifndef _WIN32
            #define _WIN32
        #endif
        #ifndef WIN32
            #define WIN32
        #endif
        #ifndef PLATFORM_32
            #define PLATFORM_32
        #endif
    //if WIN32 not defined and any of _WIN64 or WIN64 is defined, ensure both are defined
    #elif defined(_WIN64) || defined(WIN64)
        #ifndef _WIN64
            #define _WIN64
        #endif
        #ifndef WIN64
            #define WIN64
        #endif
        #ifndef PLATFORM_64
            #define PLATFORM_64
        #endif
    #endif

    //Microsoft Visual Studio warnings - get rid of unneccesary compiler warnings
    #ifdef _MSC_VER
        #pragma warning (disable:4100) //Warning Level 4: unreferenced formal parameter              : if you had to pick one useless warning to be removed from the MSVC compiler, this would be it - OK for one or more function arguments to go unreferenced!
    #endif

    //unless explicitly declared, assume we're compiling support for at least Win95 or NT applications
    #ifndef WINVER
        //enable features for Win95, NT4 and later
        #define WINVER          0x0400
    #endif
    #ifndef _WIN32_WINNT
        //features available for NT4 and later
        #define _WIN32_WINNT    0x0400
    #endif

    //since tchar.h doesn't take care of this, we'll ensure both UNICODE and both WIDE defines are declared
    //  under Windows if either _UNICODE, UNICODE or our OS independent WIDE is defined
    #if defined(_UNICODE) || defined(UNICODE) || defined(WIDE)
        #ifndef _UNICODE
            #define _UNICODE
        #endif
        #ifndef UNICODE
            #define UNICODE
        #endif
        #ifndef _WIDE
            #define _WIDE
        #endif
        #ifndef WIDE
            #define WIDE
        #endif
    #endif

    //define STRICT when not overridden by NO_STRICT - this causes windows headers to
    //      declare types that are more-portable with other versions of windows and subject to more rigorous type checks.
    #if !defined(STRICT) && !defined(NO_STRICT)
        #define STRICT
    #endif

    //declare WIN32_LEAN_AND_MEAN under WIN32 so prevent windows.h from including everything under the sun.
    //  This way, we can explicitly include the Windows headers we want and speeding up the compilation process.
    #if !defined(WIN32_LEAN_AND_MEAN) && !defined(NO_WIN32_LEAN_AND_MEAN)
        #define WIN32_LEAN_AND_MEAN
    #endif

    //include standard windows header unless specified otherwise
    #ifndef NO_WINDOWS_H
        #include <windows.h>
    #endif
    
    //unified unicode/ansi string functions should also be included by default unless specified otherwise
    #ifndef NO_TCHAR_H
        #include <tchar.h>
    #endif
#endif
    
//in WINDOWS DEBUG builds, compile-in support for CRT memory leak debugging unless NO_CRT_DEBUG is defined
#if defined(DEBUG) && !defined(NO_CRT_DEBUG) && defined(WINDOWS)
    #ifndef _CRTDBG_MAP_ALLOC
        #define _CRTDBG_MAP_ALLOC
    #endif
#endif

//include the standard C library headers and commonly used functions by default unless NO_STDLIB_H is defined
#ifndef NO_STDLIB_H
    #include <stdlib.h>
    #include <stdio.h>

    //large file support under Win32 consists of only swapping the relevant io.h functions
    //  this must be done after its initial inclusion
    #if defined(WINDOWS) && !defined(NO_64_SUPPORT) && defined(PLATFORM_32)
        //extended 64-bit file functins only available in VS 2005 CRT and up
        //  NOTE: fileoffset_t isn't a standard type, but I added it for portability to match whatever type is needed for current fseek/ftell family of functions
        #if _MSC_VER >= 1400
            #include <io.h>
            #define lseek   _lseeki64
            #define fseek   _fseeki64
            #define tell    _telli64
            #define ftell   _ftelli64
            
            //generally apps should use filesize_t whenever possible, this is only to be used
            //  when large file support is needed, but you are using the (32-bit long) versions 
            //  of the functions above
            #define fileoffset_t unsigned __int64
        #else
            //below VC 2005, fileoffset_t needs to be a long
            #define fileoffset_t long
        #endif
    #endif
    
    #ifndef _CRTDBG_MAP_ALLOC
        #define DEFAULT_CRTDBG_FLAGS     0
    #else

        //we need to include malloc.h and crtdebug.h when CRT memory checking is enabled in this order; if we don't do this and these
        //  files are included later (either implcitly or explicitly), the macros defined in crtdbg.h (like free) conflict with the desired
        //  definitions declared earlier causing errors like this:
        //      "error C2733: second C linkage of overloaded function '_calloc_dbg' not allowed"
        //  so the problem is addressed by forcing the affected includes here
        #include <malloc.h>
        #include <crtdbg.h>

        //from http://msdn2.microsoft.com/en-us/magazine/cc301382.aspx - tip #40:
        //now: https://web.archive.org/web/20080305155133/http://msdn2.microsoft.com/en-us/magazine/cc301382.aspx
        //author: John Robbins / From the December 2000 issue of MSDN Magazine
        //based on date, this likely applied to VC6, but the workaround still needed as of MSVC 7.1
        //
        //      "Don't you just hate it when you turn on memory leak detection in the C runtime and all of your leaks allocated by the
        //      new operator come out CRTDBG.H and not where you allocated memory? That drives me nuts! The problem is that there is a
        //      bug in CRTDBG.H in that the new operator is declared as an inline function. Since debug builds turn all inlining off,
        //      the new operator becomes another function and the __FILE__ macro expands to CRTDBG.H."
        //
        //      Drawback to this approach is that this causes problems in precompiled headers, as well as other source files in the
        //      same linkage unit that include this file inconsistently or differently than with other source files.
        //
        //      If you have custom new operators for a class, you will also get errors so you must undefine new before declaring your
        //      class and perform the specific new defines after your class.  You should also include a placement operator version
        //      of new in your class that matches the one in CRTDBG.H so you can get the source and line information."
        //
        #ifndef NO_CRTDBG_NEW_WORKAROUND
            #define NEW_INLINE_WORKAROUND new (_NORMAL_BLOCK,__FILE__,__LINE__)
            #define new NEW_INLINE_WORKAROUND
        #endif

        //CRT Memory Debugging Flags
        #define     DEFAULT_CRTDBG_FLAGS     (_CRTDBG_ALLOC_MEM_DF|_CRTDBG_LEAK_CHECK_DF)
    #endif
#endif

#ifndef NULL
    #define NULL 0
#endif

//unless NO_STD_TYPES is declared, always declare base types
#ifndef NO_STD_TYPES

    #ifndef NO_64_SUPPORT
        #ifdef _MSC_VER
            //Microsoft Compiler Specific
            typedef __int64                 int64;
            typedef unsigned __int64        uint64;
            typedef unsigned __int64        qword;
        #else
            //ISO C99, C89 support for 64-bit types
            //  with GCC, don't compile GCC with -pedantic otherwise you get
            //  "ISO C++ does not support 'long long'"
            typedef long long int           int64;
            typedef unsigned long long int  uint64;
            typedef unsigned long long int  qword;
        #endif
    #endif
    
    typedef signed long         int32;
    typedef signed short        int16;
    typedef signed char         int8;

    typedef unsigned long       uint32;
    typedef unsigned short      uint16;
    typedef unsigned char       uint8;

    typedef unsigned long       dword;
    typedef unsigned short      word;
    typedef unsigned char       byte;
    typedef unsigned int        uint;
    
    //in general, we want 64 bit native type to hold a file's size unless all 64-bit types
    //  have been disabled
    #if !defined(NO_64_SUPPORT) && (defined(PLATFORM_32) || defined(PLATFORM_64))
        typedef qword filesize_t;
    #else
        //unknown platform type - just use 32 bits??
        typedef dword filesize_t;
    #endif
#endif


#ifndef NO_RESULTCODES

    //code success/failure test MACROS - all failure codes are negative
    #define CODE_FAILED(code)           ((code)<0)
    #define CODE_SUCCEEDED(code)        ((code)>=0)

    typedef int ResultCode;

    enum
    {
        //generic success codes
        CODE_FOUND                      = 1,
        CODE_SUCCESS                    = 0,
        CODE_ERROR_ARGUMENTS            = -7,
        CODE_ERROR_ALLOCATION           = -4,
        CODE_ERROR_INVALID              = -11,
        CODE_ERROR_INTERNAL             = -12,
        CODE_ERROR_UNSUPPORTED          = -14,
        CODE_ERROR_NOT_FOUND            = -15,
        CODE_ERROR_OVERFLOW             = -20,
        CODE_ERROR_SYNTAX               = -22,
        CODE_ERROR_LIMIT_REACHED        = -25,
        CODE_ERROR_RANGE                = -29,
        CODE_ERROR_BUFFER_SIZE          = -30,
        CODE_ERROR_API                  = -33,
        CODE_ERROR_FILE_OPEN            = -51,
        CODE_ERROR_FILE_READ            = -56,
        CODE_ERROR_FILE_QUERY           = -58,
    };

#endif //#ifndef NO_LL_RESULTCODES

#ifndef NO_LL_MACROS

    //general purpose utility macros
    #define ZEROMEM(var)                                            memset(&(var),0,sizeof(var))
    #define MAXLEN(array)                                           (sizeof(array)/sizeof(*(array)))
    #define IS_ODD(num)                                             ((num) & 0x01)
    #define IS_EVEN(num)                                            (!IS_ODD(num))
    #define TO_LOWER(ch)                                            (( !((ch)>='A' && (ch)<='Z') ) ? (ch) : (ch)+'a'-'A')
    #define TO_UPPER(ch)                                            (( !((ch)>='a' && (ch)<='z') ) ? (ch) : (ch)-('a'-'A'))
    #define IS_WS(ch)                                               ( ' '==(ch) || '\n'==(ch) || '\r'==(ch) || '\t'==(ch) )
    #define HAS_BITS(bits,pattern)                                  (((bits)&(pattern))==(pattern))
    #define SMALLEST(x,y)                                           ((x)>(y)?(y):(x))

    #define SZLF        TEXT("\n")
    #define BACKSLASH   TEXT('\\')
    #define FRONTSLASH  TEXT('/')
    #define SPACE       TEXT(' ')
    #define LF          TEXT('\n')
    #define CR          TEXT('\r')
    #define ESC         TEXT('\x01B')   //ASCII 27
    #define PATHSLASH   BACKSLASH       //on Linux, its a frontslash

#endif //#ifndef NO_MACROS

//
// PE spec values not defined for us by Windows
//
#ifndef IMAGE_FILE_MACHINE_ARMNT
#define IMAGE_FILE_MACHINE_ARMNT 0x01C4
#endif
#ifndef IMAGE_FILE_MACHINE_ARM64
#define IMAGE_FILE_MACHINE_ARM64 0xAA64
#endif

//
// Standard function prototypes
//
int                 resultCodeToExitCode            (int code);
const TCHAR*        getFilenamePtr                  (const TCHAR* pszFilePath);
ResultCode          stringToInt32                   (const TCHAR* pszNum, int32* puRet, bool bForceDefaultBase=false, dword uDefaultBase=10, size_t uDataLen=0);
ResultCode          stringToDword                   (const TCHAR* pszNum, dword* puRet, bool bForceDefaultBase=false, dword uDefaultBase=10, size_t uDataLen=0);

size_t              stringCopy                      (TCHAR* pszDest, size_t uMaxLenDest, const TCHAR* pszSource, size_t uCopyChars=0);
ResultCode          stringAnsiToWide                (const char* pszSource, WCHAR* pszDest, size_t uMaxLenDest, size_t uCopyLen);
ResultCode          stringAnsiToWideInPlace         (char* pszDest, size_t uMaxSizeDest, size_t uCopyLen=0);

const TCHAR*        peGetMachineString              (WORD uMachine);
DWORD               peRoundUpToSectionAlignment     (DWORD dwSectionAlignment, DWORD dwVal);

#include <assert.h>
//#define assert(exp) (void)(0) //todo; swap this with standard assert macro


#endif //_STDINC_H_
